/*
speciesgen
Copyright (C) 2022-2023  prisixia

This file is part of speciesgen.

speciesgen is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

speciesgen is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with speciesgen.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "starbound/metadata.h"

namespace Starbound {

//----------------------------------------------------------------------
// Metadata
//----------------------------------------------------------------------

Metadata::Metadata(std::string name, std::string friendlyName,
				   std::string author, std::string description,
				   std::string version, std::string link,
				   std::vector<std::string> _requires)
	: m_name(std::move(name)), m_friendlyName(std::move(friendlyName)),
	  m_author(std::move(author)), m_description(std::move(description)),
	  m_version(std::move(version)), m_link(std::move(link)),
	  m_requires(std::move(_requires)), m_priority(0) {}

std::string Metadata::GetName() const { return m_name; }

std::string Metadata::GetFriendlyName() const { return m_friendlyName; }

std::string Metadata::GetDescription() const { return m_description; }

std::string Metadata::GetAuthor() const { return m_author; }

std::string Metadata::GetVersion() const { return m_version; }

std::string Metadata::GetLink() const { return m_link; }

std::string Metadata::GetSteamContentID() const { return m_steamContentId; }

std::string Metadata::GetTags() const { return m_tags; }

std::vector<std::string> Metadata::GetIncludes() const { return m_includes; }

std::vector<std::string> Metadata::GetRequires() const { return m_requires; }

uint32_t Metadata::GetPriority() const { return m_priority; }

void Metadata::SetName(const std::string &name) { m_name = name; }

void Metadata::SetFriendlyName(const std::string &friendlyName) {
	m_friendlyName = friendlyName;
}

void Metadata::SetDescription(const std::string &description) {
	m_description = description;
}

void Metadata::SetAuthor(const std::string &author) { m_author = author; }

void Metadata::SetVersion(const std::string &version) { m_version = version; }

void Metadata::SetLink(const std::string &link) { m_link = link; }

void Metadata::SetSteamContentID(const std::string &steamContentId) {
	m_steamContentId = steamContentId;
}

void Metadata::SetTags(const std::string &tags) { m_tags = tags; }

void Metadata::SetIncludes(const std::vector<std::string> &includes) {
	m_includes = includes;
}

void Metadata::SetRequires(const std::vector<std::string> &_requires) {
	m_requires = _requires;
}

void Metadata::SetPriority(const uint32_t priority) { m_priority = priority; }
} // namespace Starbound
